# -*- coding: utf-8 -*-
'''
Module: num2word_FA.py
Requires: num2word_EU.py
Version: 1.2

Author:
   Taro Ogawa (tso@users.sourceforge.org)
   
Copyright:
    Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.

Licence:
    This module is distributed under the Lesser General Public Licence.
    http://www.opensource.org/licenses/lgpl-license.php

Data from:
    http://www.uni-bonn.de/~manfear/large.php
    
Usage:
    from num2word_FA import n2w, to_card, to_ord, to_ordnum
    to_card(1234567890)
    n2w.is_title = True
    to_card(1234567890)
    to_ord(1234567890)
    to_ordnum(1234567890)
    to_year(1976)
    to_currency(dollars*100 + cents, longval=False)
    to_currency((dollars, cents))
    

History:
    1.2: to_ordinal_num() made shorter and simpler (but slower)
         strings in merge() now interpolated
         to_year() and to_currency() added
         
    1.1: to_ordinal_num() fixed for 11,12,13   
'''

from . import num2word_EU


class Num2Word_FA(num2word_EU.Num2Word_EU):
    def set_high_numwords(self, high):
        max = 3 + 3 * len(high)
        for word, n in zip(high, list(range(max, 3, -3))):
            self.cards[10 ** n] = word + "میلیون"

    def set_high_numwords_str(self, high):
        max = 3 + 3 * len(high)
        for word, n in zip(high, list(range(max, 3, -3))):
            self.cards_str[10 ** n] = word + "میلیون"

    def setup(self):
        self.negword = "منفی "
        self.pointword = "نقطه"
        self.errmsg_nonnum = "تنها اعداد می توانند به حروف تبدیل شوند."
        self.exclude_title = ["و", "نقطه", "منفی"]

        self.mid_numwords_str = [(1000, "هزار"), (900, "نهصد"), (800, "هشتصد"), (700, "هفتصد"), (600, "ششصد"),
                                 (500, "پانصد"), (400, "چهارصد"), (300, "سیصد"), (200, "دویست"), (100, "یکصد"),
                                 (90, "نود"), (80, "هشتاد"), (70, "هفتاد"),
                                 (60, "شصت"), (50, "پنجاه"), (40, "چهل"),
                                 (30, "سی"), (20, "بیست"), (10, "ده")]
        self.mid_numwords = [(1000, "هزار"), (900, "نهصد"), (800, "هشتصد"), (700, "هفتصد"), (600, "ششصد"),
                             (500, "پانصد"), (400, "چهارصد"), (300, "سیصد"), (200, "دویست"), (100, "صد"), (90, "نود"),
                             (80, "هشتاد"), (70, "هفتاد"), (60, "شصت"), (50, "پنجاه"), (40, "40"), (30, "30"),
                             (20, "بیست"), (10, "ده")]
        self.low_numwords_str = ["بیست", "نوزده", "هجده", "هفده",
                             "شانزده", "پانزده", "چهارده", "سیزده",
                             "دوازده", "یازده", "ده", "نه", "هشت",
                             "هفت", "شش", "پنج", "چهار", "سه", "دو",
                             "یک", "صفر"]
        self.low_numwords = ["20", "19", "18", "17",
                             "16", "15", "14", "13",
                             "12", "11", "10", "9", "8",
                             "7", "6", "5", "4", "3", "2",
                             "1", "0"]

        self.ords = {"یک": "اول",
                     "دو": "دوم",
                     "سه": "سوم",
                     "چهار": "چهارم",
                     "پنج": "پنجم",
                     "شش": "ششم",
                     "هفت": "هفتم",
                     "هشت": "هشتم",
                     "نه": "نهم",
                     "ده": "دهم",
                     "دوازده": "دوازدهم"}

    def merge(self, curr, next):
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1 and nnum < 100:
            return next
        elif 100 > cnum > nnum:
            return ("%s و %s" % (ctext, ntext), cnum + nnum)
        elif cnum >= 100 > nnum:
            return ("%s و %s" % (ctext, ntext), cnum + nnum)
        elif nnum > cnum:
            return ("%s %s" % (ctext, ntext), cnum * nnum)
        return ("%s و %s" % (ctext, ntext), cnum + nnum)

    def merge_str(self, curr, next):
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1 and nnum < 100:
            return next
        elif 100 > cnum > nnum:
            return ("%s و %s" % (ctext, ntext), cnum + nnum)
        elif cnum >= 100 > nnum:
            return ("%s و  %s" % (ctext, ntext), cnum + nnum)
        elif nnum > cnum:
            return ("%s %s" % (ctext, ntext), cnum * nnum)
        return ("%s و %s" % (ctext, ntext), cnum + nnum)

    def to_ordinal(self, value):
        self.verify_ordinal(value)
        outwords = self.to_cardinal(value).split(" ")
        lastwords = outwords[-1].split(" ")
        lastword = lastwords[-1].lower()
        try:
            lastword = self.ords[lastword]
        except KeyError:
            if lastword[-1] == "l":
                lastword = lastword[:-1] + "ie"
            lastword += "م"
        lastwords[-1] = self.title(lastword)
        outwords[-1] = " و ".join(lastwords)
        return " ".join(outwords)

    def to_ordinal_num(self, value):
        self.verify_ordinal(value)
        return "%s%s" % (value, self.to_ordinal(value)[-2:])

    def to_year(self, val, longval=True):
        if not (val // 100) % 10:
            return self.to_cardinal(val)
        return self.to_splitnum(val, hightxt="صد", jointxt="و",
                                longval=longval)

    def to_currency(self, val, longval=True):
        return self.to_splitnum(val, hightxt="تومان", lowtxt="قران",
                                jointxt="و", longval=longval)


n2w = Num2Word_FA()
to_card = n2w.to_cardinal
to_card_str = n2w.to_cardinal_str
to_ord = n2w.to_ordinal
to_ordnum = n2w.to_ordinal_num
to_year = n2w.to_year


def main():
    for val in [1, 11, 12, 21, 31, 33, 71, 80, 81, 91, 99, 100, 101, 102, 155,
                180, 300, 308, 832, 1000, 1001, 1061, 1100, 1500, 1701, 3000,
                8280, 8291, 150000, 500000, 1000000, 2000000, 2000001,
                -21212121211221211111, -2.121212, -1.0000100]:
        n2w.test(val)
    n2w.test(
        1325325436067876801768700107601001012212132143210473207540327057320957032975032975093275093275093270957329057320975093272950730)
    for val in [1, 120, 1000, 1120, 1800, 1976, 2000, 2010, 2099, 2171]:
        print((val, "هست", n2w.to_currency(val)))
        print((val, "هست", n2w.to_year(val)))


if __name__ == "__main__":
    main()



# Generated by Django 2.2 on 2019-06-15 14:30

import ckeditor_uploader.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0005_auto_20181210_1511'),
    ]

    operations = [
        migrations.AlterField(
            model_name='news',
            name='categories',
            field=models.ManyToManyField(blank=True, to='categories.Category', verbose_name='دسته بندی ها'),
        ),
        migrations.AlterField(
            model_name='news',
            name='content',
            field=ckeditor_uploader.fields.RichTextUploadingField(verbose_name='محتوا'),
        ),
        migrations.AlterField(
            model_name='news',
            name='publish_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='تاریخ انتشار'),
        ),
        migrations.AlterField(
            model_name='news',
            name='title',
            field=models.CharField(max_length=200, verbose_name='عنوان'),
        ),
        migrations.AlterField(
            model_name='news',
            name='user_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='کاربر'),
        ),
    ]



from django.conf.urls import include, url

from aparnik.contrib.province.api.views import ProvinceListAPIView, ProvinceCreateAPIView, ProvinceDetailAPIView

app_name = 'province'

urlpatterns = [
    url(r'^$', ProvinceListAPIView.as_view(), name='list'),
    url(r'^create/$', ProvinceCreateAPIView.as_view(), name='create'),
    url(r'^(?P<id>\d+)/$', ProvinceDetailAPIView.as_view(), name='detail'),
    url(r'^(?P<province_id>\d+)/city/', include('aparnik.contrib.province.api.urls-city', namespace='city-api')),
    url(r'^(?P<province_id>\d+)/shahrak/', include('aparnik.contrib.province.api.urls-shahrak', namespace='shahrak-api')),
]


# -*- coding: utf-8 -*-


from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)

from .serializers import VoucherListSerializer
from ..models import Voucher


class VoucherListAPIView(ListAPIView):
    serializer_class = VoucherListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        queryset = Voucher.objects.this_user(user, only_accessible=False)
        # if 'product_id' in self.request.parser_context['kwargs']:
        #     product_id = self.request.parser_context['kwargs']['product_id']
        #     queryset = Voucher.objects.this_user(user, product_id)
        return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-14 13:30


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0011_remove_basesegment_sort'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pagesort',
            options={'ordering': ['sort'], 'verbose_name': 'Page Sort', 'verbose_name_plural': 'Pages Sort'},
        ),
        migrations.AlterModelOptions(
            name='segmentsort',
            options={'ordering': ['sort'], 'verbose_name': 'Segment Sort', 'verbose_name_plural': 'Segments Sort'},
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf import settings
from django.contrib import admin

from .models import Information
from aparnik.contrib.basemodels.admin import BaseModelAdmin
# from socials.models import SocialNetwork

# Register your models here.


# class InlineSocialNetwork(admin.StackedInline):
#     extra = 0
#     model = SocialNetwork


class InformationAdmin(BaseModelAdmin):
    fields = ['website', 'address', 'email', 'phone', 'about_us', 'image', 'is_active', 'socials',
              'short_blogs', 'slider_segment_obj']
    list_display = ['website', 'phone', 'about_us', 'is_active']
    list_filter = ['website', 'is_active']
    search_fields = ['address', 'phone', 'about_us', 'website']
    exclude = []
    raw_id_fields = ['slider_segment_obj']
    # inlines = [InlineSocialNetwork]

    def __init__(self, *args, **kwargs):
        Klass = InformationAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Information


admin.site.register(Information, InformationAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-30 20:25


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0008_product_further_details'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='aparnik_bon_return_expire_value',
            field=models.PositiveIntegerField(default=0, help_text='If the number is set to 0, then the value in the settings will apply', verbose_name='Bon Return Expire ( Hours )'),
        ),
        migrations.AddField(
            model_name='product',
            name='aparnik_bon_return_value',
            field=models.PositiveIntegerField(default=0, help_text='If the number is set to 0, then the value in the settings will apply', verbose_name='Bon Return'),
        ),
        migrations.AddField(
            model_name='product',
            name='has_permit_use_aparnik_bon_value',
            field=models.NullBooleanField(help_text='If the number is set to 0, then the value in the settings will apply', verbose_name='Has permit use aparnik bon'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Q, Count
from django.contrib.auth import get_user_model
from django.core.validators import ValidationError, MinValueValidator, MaxValueValidator
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now
import uuid
from decimal import Decimal

from aparnik.utils.utils import field_with_prefix
from aparnik.utils.fields import *
from aparnik.settings import aparnik_settings

User = get_user_model()


# Create your models here.
# TODO: add validation for limited user custom user or all user when add
class CouponManager(models.Manager):

    def get_queryset(self):
        return super(CouponManager, self).get_queryset()

    def status(self, code, user, order):

        try:

            coupon = self.get_queryset().get(code=code)
        except:

            raise ValidationError({'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_NOT_FOUND)]})

        # check min and max order
        if order.status != order.STATUS_WAITING:

            raise ValidationError({'coupon': [_('This order finished before.')]})
        if coupon.min_price_order > 0 and coupon.min_price_order > order.get_total_cost_order():

            raise ValidationError({'coupon': [
                Coupon.get_status_description(status_title=Coupon.COUPON_STATUS_ORDER_MINIMUM_IS_REQUIRED)]})

        if coupon.max_price_order > 0 and coupon.max_price_order < order.get_total_cost_order():

            raise ValidationError(
                {'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_ORDER_MAXIMUM_IS_REQUIRED)]})

        # check expire date
        if coupon.expire_at is not None and coupon.expire_at < now():

            raise ValidationError({'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_EXPIRED)]})

        # add coupon
        if coupon.for_users == Coupon.COUPON_FOR_ALL_USER and coupon.membership_set.filter(user=user,
                                                                                           is_redeem=True).count() == 0:

            return Coupon.COUPON_STATUS_OK
        elif coupon.for_users == Coupon.COUPON_FOR_CUSTOM_USER and coupon.membership_set.filter(user=user,
                                                                                                is_redeem=False).count() > 0:

            return Coupon.COUPON_STATUS_OK
        elif coupon.for_users == Coupon.COUPON_FOR_LIMITED_USER and coupon.membership_set.filter(
                is_redeem=True).count() <= coupon.for_limited_users_value:

            return Coupon.COUPON_STATUS_OK

        raise ValidationError({'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_EXPIRED)]})

    def add_coupon(self, code, user, order):

        if Coupon.COUPON_STATUS_OK == self.status(code=code, user=user, order=order):
            try:

                coupon = self.get_queryset().get(code=code)
            except:

                raise ValidationError({'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_NOT_FOUND)]})

            if Membership.objects.filter(user=user, coupon=coupon).count() == 0:
                membership = Membership.objects.create(user=user, coupon=coupon)
                membership.save()

            return coupon

        raise ValidationError({'coupon': [Coupon.get_status_description(Coupon.COUPON_STATUS_EXPIRED)]})


class Coupon(models.Model):
    # coupon status
    COUPON_STATUS_OK = 'o'
    COUPON_STATUS_REDEEM = 'r'
    COUPON_STATUS_EXPIRED = 'e'
    COUPON_STATUS_NOT_FOUND = 'nf'
    COUPON_STATUS_ORDER_MINIMUM_IS_REQUIRED = 'ominr'
    COUPON_STATUS_ORDER_MAXIMUM_IS_REQUIRED = 'omaxr'
    COUPON_STATUS_CHOICES = (
        (COUPON_STATUS_OK, _('Ok')),
        (COUPON_STATUS_REDEEM, _('The code was redeem.')),
        (COUPON_STATUS_EXPIRED, _('The code expired.')),
        (COUPON_STATUS_NOT_FOUND, _('The code does not exist.')),
        (COUPON_STATUS_ORDER_MINIMUM_IS_REQUIRED, _('Minimum price for order must greater than your order price.')),
        (COUPON_STATUS_ORDER_MAXIMUM_IS_REQUIRED, _('Maximum price for order must less than your order price.')),
    )

    COUPON_TYPE_PERCENT = 'p'
    COUPON_TYPE_VALUE = 'v'
    COUPON_TYPE_CHOICES = (
        (COUPON_TYPE_PERCENT, _('Percent')),
        (COUPON_TYPE_VALUE, _('Value')),

    )
    COUPON_FOR_ALL_USER = 'a'
    COUPON_FOR_LIMITED_USER = 'l'
    COUPON_FOR_CUSTOM_USER = 'c'
    COUPON_FOR_CHOICES = (
        (COUPON_FOR_ALL_USER, _('All user')),
        (COUPON_FOR_LIMITED_USER, _('Limited user')),
        (COUPON_FOR_CUSTOM_USER, _('Custom user')),
    )

    users = models.ManyToManyField(User, through='Membership', verbose_name=_('Membership'))
    code = models.CharField(max_length=10, blank=True, null=True, unique=True, verbose_name=_('Code'), help_text=_('If blank default no limitation.'))
    expire_at = models.DateTimeField(blank=True, null=True, verbose_name=_('Expire at'), help_text=_('If blank default no limitation.'))
    value = PriceField(validators=[MinValueValidator(1)], verbose_name=_('Value'))
    type = models.CharField(max_length=2, choices=COUPON_TYPE_CHOICES, default=COUPON_TYPE_PERCENT, verbose_name=_('type'))
    max_price_order = PriceField(big=True, default=0, verbose_name=_('Max price of order'), help_text=_('If blank default no limitation.')) # hadaksar mablaghe factor
    min_price_order = PriceField(big=True, default=0, verbose_name=_('Min price of order'), help_text=_('If blank default no limitation.')) # hadaghal mablaghe factor
    for_users = models.CharField(max_length=2, choices=COUPON_FOR_CHOICES, default=COUPON_FOR_ALL_USER, verbose_name=_('For users'))
    for_limited_users_value = models.IntegerField(blank=True, null=True, verbose_name=_('Limited users value'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = CouponManager()

    def clean(self, *args, **kwargs):
        super(Coupon, self).clean(*args, **kwargs)
        # Don't allow complete action entries.
        self.clean_fields()
        if self.type == Coupon.COUPON_TYPE_PERCENT and self.value > Decimal(100):
            raise ValidationError({'value': [_('Value cannot greater than 100.')]})

        if self.code is None:
            def generate():
                base = 4
                coupon_code = uuid.uuid4().__str__().replace('-', '')

                coupon_code = coupon_code[base:base + aparnik_settings.COUPON_CODE_LENGTH]
                if Coupon.objects.filter(code=coupon_code).count() > 0:
                    return generate()
                return coupon_code

            self.code = generate()

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Coupon, self).save(*args, **kwargs)

    def __str__(self):
        return self.code

    class Meta:
        ordering = ('-created_at',)
        verbose_name = _('Coupon')
        verbose_name_plural = _('Coupons')

    @staticmethod
    def get_status_description(status_title):

        for status in Coupon.COUPON_STATUS_CHOICES:
            if status[0] == status_title:
                return status[1]

        return None

    def status(self, user, order):
        # TODO: handle for user
        return Coupon.COUPON_STATUS_OK

    def calculate_discount(self, total_cost):

        if self.type == self.COUPON_TYPE_PERCENT:
            return total_cost * (self.value / Decimal('100'))
        # elif self.type == self.COUPON_TYPE_VALUE:
        else:
            return self.value

    def redeem(self, user):
        membership = Membership.objects.filter(user=user, coupon=self, is_redeem=False).first()
        membership.is_redeem = True
        membership.save()

    @staticmethod
    def sort_redeem(return_key='redeem', prefix=''):
        sort = {
            return_key: {
                'label': 'مصرف',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(
                            field_with_prefix('pk', prefix=prefix),
                            filter=Q(**{field_with_prefix('is_redeem', prefix=prefix): True})
                        )
                },
                'key_sort': 'sort_count',
            }
        }
        return sort


class CouponAllUsersManager(CouponManager):

    def get_queryset(self):
        return super(CouponAllUsersManager, self).get_queryset().filter(for_users=Coupon.COUPON_FOR_ALL_USER)


class CouponForAllUser(Coupon):

    objects = CouponAllUsersManager()

    class Meta:
        proxy = True
        verbose_name = _('Coupon for all users')
        verbose_name_plural = _('Coupons for all users')

    def save(self, *args, **kwargs):
        self.full_clean()
        self.for_users = Coupon.COUPON_FOR_ALL_USER
        return super(CouponForAllUser, self).save(*args, **kwargs)


class CouponLimitedUsersManager(CouponManager):

    def get_queryset(self):
        return super(CouponLimitedUsersManager, self).get_queryset().filter(for_users=Coupon.COUPON_FOR_LIMITED_USER)


class CouponForLimitedUser(Coupon):

    objects = CouponLimitedUsersManager()

    class Meta:
        proxy = True
        verbose_name = _('Coupon for limited users')
        verbose_name_plural = _('Coupons for limited users')

    def clean(self, *args, **kwargs):
        super(CouponForLimitedUser, self).clean(*args, **kwargs)
        if self.for_limited_users_value is None:
            raise ValidationError({'for_limited_users_value': [_('This field is requeired.')]})

    def save(self, *args, **kwargs):
        self.full_clean()
        self.for_users = Coupon.COUPON_FOR_LIMITED_USER
        return super(CouponForLimitedUser, self).save(*args, **kwargs)


class CouponCustomUsersManager(CouponManager):

    def get_queryset(self):
        return super(CouponCustomUsersManager, self).get_queryset().filter(for_users=Coupon.COUPON_FOR_CUSTOM_USER)


class CouponForCoustomUser(Coupon):

    objects = CouponCustomUsersManager()

    class Meta:
        proxy = True
        verbose_name = _('Coupon for custom users')
        verbose_name_plural = _('Coupons for custom users')

    def save(self, *args, **kwargs):
        self.full_clean()
        self.for_users = Coupon.COUPON_FOR_CUSTOM_USER
        return super(CouponForCoustomUser, self).save(*args, **kwargs)


# Membership
class Membership(models.Model):
    user = models.ForeignKey(User, related_name='coupons', on_delete=models.CASCADE, verbose_name=_('User'))
    coupon = models.ForeignKey(Coupon, on_delete=models.CASCADE, verbose_name=_('Coupon'))
    is_redeem = models.BooleanField(default=False, verbose_name=_('Is redeem'))

    class Meta:
        verbose_name = _('Membership')
        verbose_name_plural = _('Memberships')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-14 17:14


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('basemodels', '0010_basemodel_visit_count'),
    ]

    operations = [
        migrations.CreateModel(
            name='Counter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.CharField(choices=[('v', 'Visiting'), ('s', 'Sharing')], max_length=2, verbose_name='Action kind')),
                ('create_date', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('model_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='basemodels.BaseModel', verbose_name='Base Model')),
                ('user_obj', models.ForeignKey(blank=True, default=0, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Counter',
                'verbose_name_plural': 'Counters',
            },
        ),
    ]



from django.conf import settings


class WebFactionFixes:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        self.process_request(request)

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response

    def process_request(self, request):
        if 'HTTP_X_FORWARDED_FOR' in request.META:
            ip = request.META['HTTP_X_FORWARDED_FOR'].split(",")[0].strip()
            request.META['REMOTE_ADDR'] = ip


class DisableCSRFMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        self.process_request(request)

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response

    def process_request(self, req):
        attr = '_dont_enforce_csrf_checks'
        if settings.DEBUG and not getattr(req, attr, False):
            setattr(req, attr, True)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-11 16:40


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('socials', '0004_auto_20181203_1328'),
    ]

    operations = [
        migrations.AddField(
            model_name='socialnetwork',
            name='value',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Value'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-25 10:57


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='setting',
            name='is_variable_in_home',
            field=models.BooleanField(default=False, verbose_name='Is variable show in home'),
        ),
        migrations.AlterField(
            model_name='setting',
            name='value_type',
            field=models.CharField(choices=[('s', 'String'), ('i', 'Int'), ('b', 'Boolean'), ('dt', 'Date Time'), ('fr', 'Function setting return value')], max_length=31, verbose_name='Value Type'),
        ),
    ]



from django.conf.urls import url, include

from .views import NotifyMeListAPIView, NotifyMeSetAPIView

app_name = 'notifiesme'

urlpatterns = [
    url(r'^$', NotifyMeListAPIView.as_view(), name='list'),
    url(r'^(?P<model_id>\d+)/set/$', NotifyMeSetAPIView.as_view(), name='set'),
    # url(r'^model/(?P<model_id>\d+)/set/$', BookmarkSetAPIView.as_view(), name='set'),

]


# Generated by Django 2.2 on 2019-06-15 14:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0011_auto_20190428_1107'),
    ]

    operations = [
        migrations.AlterField(
            model_name='orderitem',
            name='product_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='orderitem_set', to='products.Product', verbose_name='Product'),
        ),
    ]



from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer

from ..models import Bookmark
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from aparnik.settings import aparnik_settings

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


# Bookmark List Serializer
class BookmarkListSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    model = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BookmarkListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = [
            'url',
            'model'
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_model(self, obj):
        return ModelListPolymorphicSerializer(obj.model_obj, many=False, read_only=True, context=self.context).data


# Bookmark Details Serializer
class BookmarkDetailSerializer(BookmarkListSerializer):
    user = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BookmarkDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = BookmarkListSerializer.Meta.fields + [
            'user'
        ]

    def get_user(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data


from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.api.serializers import ModelSerializer

from ..models import *


class SupportDetailsSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    socials = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SupportDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Support
        fields = [
            'id',
            'url',
            'title',
            'online',
            'offline',
            'phone',
            'socials',
            'online_days',
        ]

    def get_image(self, obj):
        return self.get_image_ur(obj.image)

    def get_image_ur(self, filed):
        try:
            image = self.context['request'].build_absolute_uri(filed.url)
        except:
            image = None
        return image

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_socials(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.socials, many=True, read_only=True, context=self.context).data



from django.conf.urls import include, url

from aparnik.contrib.province.api.views import CityCreateAPIView, CityDetailAPIView, CityListAPIView

app_name = 'province'

urlpatterns = [
    url(r'^$', CityListAPIView.as_view(), name='list'),
    url(r'^create/$', CityCreateAPIView.as_view(), name='create'),
    url(r'^(?P<id>\d+)/$', CityDetailAPIView.as_view(), name='detail'),
    url(r'^(?P<city_id>\d+)/town/', include('aparnik.contrib.province.api.urls-town', namespace='town-api')),
]


"""
Django settings for src project.

Generated by 'django-admin startproject' using Django 1.8.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import dj_database_url
import datetime
from django.utils.translation import ugettext_lazy as _

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('DJANGO_SECRET_KEY')
if not SECRET_KEY:
    raise ValueError('Secret key does not set')

INSTALLED_APPS = (
    'aparnik.contrib.suit',
    'debug_toolbar',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'jalali_date',  # required for admin jalali date
    'rest_framework',
    'fcm_django',
    'dynamic_raw_id',
    'django_filters',
    'ckeditor',
    'ckeditor_uploader',
    'tagulous',
    'colorfield',
    # This applications
    'aparnik',
    'aparnik.contrib.province',
    'aparnik.contrib.users',
    'aparnik.contrib.audits',
    'aparnik.contrib.addresses',
    'aparnik.contrib.bankaccounts',
    'aparnik.contrib.managements',
    'aparnik.contrib.aboutus',
    'aparnik.contrib.contactus',
    'aparnik.contrib.invitation',
    'aparnik.contrib.basemodels',
    'aparnik.contrib.filefields',
    'aparnik.contrib.bookmarks',
    'aparnik.contrib.reviews',
    'aparnik.contrib.questionanswers',
    'aparnik.contrib.socials',
    'aparnik.contrib.sliders',
    'aparnik.contrib.settings',
    'aparnik.contrib.counters',
    'aparnik.contrib.notifications',
    'aparnik.contrib.notifiesme',
    'aparnik.contrib.shortblogs',
    'aparnik.contrib.supports',
    'aparnik.contrib.faq',
    'aparnik.contrib.termsandconditions',
    'aparnik.contrib.categories',
    'aparnik.contrib.pages',
    'aparnik.contrib.segments',
    'aparnik.contrib.buttons',
    'aparnik.contrib.tickets',
    'aparnik.packages.shops.products',
    'aparnik.packages.shops.productssharing',
    'aparnik.packages.shops.zarinpals',
    'aparnik.packages.shops.payments',
    'aparnik.packages.shops.subscriptions',
    'aparnik.packages.shops.orders',
    'aparnik.packages.shops.coupons',
    'aparnik.packages.shops.cosales',
    'aparnik.packages.shops.vouchers',
    'aparnik.packages.shops.files',
    'aparnik.packages.educations.educations',
    'aparnik.packages.educations.books',
    'aparnik.packages.educations.teachers',
    'aparnik.packages.educations.courses',
    'aparnik.packages.educations.progresses',
    'aparnik.packages.news',

)

MIDDLEWARE = (
    # 'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sites.middleware.CurrentSiteMiddleware',
    # third party
    # 'DJANGO_PROJECT.middleware.WebFactionFixes',
    'testproject.middleware.DisableCSRFMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)
# ROOT_URLCONF = 'DJANGO_PROJECT.urls.common_urls'
ROOT_URLCONF = 'testproject.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')]
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': True,
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                # 'applications.context_processors.application',
                # 'cart.context_processors. cart',
            ],
        },
    },
]

# WSGI_APPLICATION = 'DJANGO_PROJECT.wsgi.application'
WSGI_APPLICATION = 'testproject.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db2.sqlite3'),
    }
}

FILE_UPLOAD_HANDLERS = [
    'django.core.files.uploadhandler.TemporaryFileUploadHandler',
    'django.core.files.uploadhandler.MemoryFileUploadHandler',
]

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/


LANGUAGE_CODE = 'fa-ir'  # 'en-us'

TIME_ZONE = 'Asia/Tehran'

USE_I18N = True

USE_L10N = True

USE_TZ = os.environ.get('USE_TZ', "True") == "True"

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale/'),
)

LANGUAGES = [
    ('fa', _('Persian')),
    # ('en', _('English')),
    # ('es', _('Espaniol')),
]

# Django Rest Framework

REST_FRAMEWORK = {
    # Attention: change it for custom handle
    # 'EXCEPTION_HANDLER': 'users.utils.custom_exception_handler',
    'DEFAULT_PERMISSION_CLASSES': (
        # 'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        # Attention: change it for custom json response
        # 'users.customJSON.CustomJSONRenderer',
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),
    'PAGE_SIZE': 25,
}

# Allow all host headers
ALLOWED_HOSTS = ['*']

# STATICFILES_DIRS = (
#     os.path.join(BASE_DIR, 'static'),
# )


JWT_AUTH = {
    'JWT_ENCODE_HANDLER':
        'rest_framework_jwt.utils.jwt_encode_handler',

    'JWT_DECODE_HANDLER':
        'rest_framework_jwt.utils.jwt_decode_handler',

    'JWT_PAYLOAD_HANDLER':
        'rest_framework_jwt.utils.jwt_payload_handler',

    'JWT_PAYLOAD_GET_USER_ID_HANDLER':
        'rest_framework_jwt.utils.jwt_get_user_id_from_payload_handler',

    'JWT_RESPONSE_PAYLOAD_HANDLER':
    # 'users.api.views.jwt_response_payload_handler',
        'aparnik.contrib.users.api.views.jwt_response_payload_handler',
    # 'rest_framework_jwt.utils.jwt_response_payload_handler',
    # 'my_jwt.views.jwt_response_payload_handler',

    'JWT_SECRET_KEY': SECRET_KEY,
    'JWT_GET_USER_SECRET_KEY': None,
    'JWT_PUBLIC_KEY': None,
    'JWT_PRIVATE_KEY': None,
    'JWT_ALGORITHM': 'HS256',
    'JWT_VERIFY': True,
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_LEEWAY': 0,
    'JWT_EXPIRATION_DELTA': datetime.timedelta(days=300),
    'JWT_AUDIENCE': None,
    'JWT_ISSUER': None,

    'JWT_ALLOW_REFRESH': False,
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(days=7),

    'JWT_AUTH_HEADER_PREFIX': 'APARNIK',
    'JWT_AUTH_COOKIE': None,

}

# Django debug toolbar
DEBUG_TOOLBAR_PATCH_SETTINGS = True
INTERNAL_IPS = ('127.0.0.1',)

CORS_ORIGIN_ALLOW_ALL = True

# CORS_ORIGIN_WHITELIST = (
#     'restzahedipro.s3.amazonaws.com',
# )
# CORS_ORIGIN_ALLOW_ALL = True
# Access-Control-Allow-Origin: *

# jalali date time
JALALI_FORMAT = os.environ.get('JALALI_FORMAT', "%Y/%m/%d")

APARNIK = {
    'AWS_ACTIVE': os.environ.get('AWS_ACTIVE', "False") == "True",
    'BANK_ACTIVE': os.environ.get('BANK_ACTIVE', "False") == "True",
    'ZARINPAL_MERCHENT_CODE': os.environ.get('ZARINPAL_MERCHENT_CODE', ''),
    # 'USER_UPDATE_SERIALIZER': 'users.api.serializers.UserUpdateSerializer',
    'USER_LOGIN_WITH_PASSWORD': os.environ.get('USER_LOGIN_WITH_PASSWORD', "True") == "True",
    # 'USER_LOGIN_API_VIEW': 'users.api.views.UserLoginAPIView',
    # 'USER_IS_SHOW_ADMIN_PANEL': False,
    'USER_REGISTER_WITH_LOGIN': os.environ.get('USER_REGISTER_WITH_LOGIN', "True") == "True",
    'API_PRODUCT_MODE': os.environ.get('DJANGO_API_PRODUCT_MODE', "True") == "True",
    'SMS_EXPIRE_TOKEN': os.environ.get('SMS_EXPIRE_TOKEN', 180),
    'SMS_API_KEY': os.environ.get('SMS_API_KEY', ''),
    'SMS_OTA_NAME': os.environ.get('SMS_OTA_NAME', ""),
    'SEND_SMS': os.environ.get('SEND_SMS', 'True') == 'True',
    'VERIFY_FAKE': os.environ.get('VERIFY_FAKE', 'False') == 'True',
    'VERIFY_FAKE_CODE': os.environ.get('VERIFY_FAKE_CODE', ''),
    'JALALI_FORMAT': JALALI_FORMAT,
}

AWS_ACTIVE = APARNIK['AWS_ACTIVE']
USER_LOGIN_WITH_PASSWORD = APARNIK['USER_LOGIN_WITH_PASSWORD']

CKEDITOR_CONFIGS = {
    'basic': {
        'toolbar': 'Basic',
    },
    'advance': {
        'toolbar': 'Advance',
    },
    'custom': {
        'toolbar': 'Custom',
        'toolbar_Custom': [
            {'name': 'clipboard', 'items': ['Cut', 'Copy', 'Paste', 'PasteText', 'PasteFromWord', '-', 'Undo', 'Redo']},
            {'name': 'editing', 'items': ['Find', 'Replace', '-', 'SelectAll']},
            {'name': 'colors', 'items': ['TextColor', 'BGColor']},
            {'name': 'styles', 'items': ['Styles', 'Format', 'Font', 'FontSize']},
            {'name': 'forms',
             'items': ['Form', 'Checkbox', 'Radio', 'TextField', 'Textarea', 'Select', 'Button', 'ImageButton',
                       'HiddenField']},
            {'name': 'basicstyles',
             'items': ['Bold', 'Italic', 'Underline', 'Strike', 'Subscript', 'Superscript', '-', 'RemoveFormat']},
            {'name': 'paragraph',
             'items': ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', '-', 'Blockquote', 'CreateDiv', '-',
                       'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock', '-', 'BidiLtr', 'BidiRtl',
                       'Language']},
            {'name': 'links', 'items': ['Link', 'Unlink', 'Anchor']},
            {'name': 'insert',
             'items': ['Table', 'HorizontalRule', 'Smiley', 'SpecialChar']},  # 'Image', 'Flash','PageBreak', 'Iframe'
        ]
    }
}

CKEDITOR_UPLOAD_PATH = "/"

TAGGIT_CASE_INSENSITIVE = True

# User
# AUTH_USER_MODEL = 'users.User'
# AUTHENTICATION_BACKENDS = ('aparnik.contrib.users.backends.AuthBackend', 'django.contrib.auth.backends.ModelBackend')
AUTH_USER_MODEL = 'aparnik_users.User'
AUTHENTICATION_BACKENDS = (
    'aparnik.contrib.users.backends.AuthBackend',
    # 'django.contrib.auth.backends.ModelBackend'
)

NOTIFICATION_API_KEY = os.environ.get('NOTIFICATION_API_KEY', None)
NOTIFICATION_APP_ID = os.environ.get('NOTIFICATION_APP_ID', None)
FCM_DJANGO_SETTINGS = {
    'FCM_SERVER_KEY': NOTIFICATION_API_KEY,
    # true if you want to have only one active device per registered user at a time
    # default: False
    'ONE_DEVICE_PER_USER': False,
    # devices to which notifications cannot be sent,
    # are deleted upon receiving error response from FCM
    # default: False
    'DELETE_INACTIVE_DEVICES': False,
}

TINYMCE_DEFAULT_CONFIG = {
    'height': 360,
    'width': 1120,
    'cleanup_on_startup': True,
    'custom_undo_redo_levels': 20,
    'selector': 'textarea',
    'theme': 'modern',
    'plugins': 'textcolor save link image media preview codesample contextmenu table code lists fullscreen  insertdatetime  nonbreaking contextmenu directionality searchreplace wordcount visualblocks visualchars code fullscreen autolink lists  charmap print  hr anchor pagebreak',
    'toolbar1': 'save fullscreen bold italic underline,fontselect,fontsizeselect  | forecolor backcolor | alignleft aligncenter alignright alignjustify'
                '| bullist numlist | outdent indent | table | link image media | codesample | preview code |',
    'contextmenu': 'formats | link image',

    'menubar': True,
    'statusbar': True,

}

# Filebrowser
FILEBROWSER_DIRECTORY = ''
DIRECTORY = ''



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model

from aparnik.contrib.settings.models import Setting

User = get_user_model()


# Create your models here.
class InviteManager(models.Manager):

    def get_queryset(self):
        return super(InviteManager, self).get_queryset()

    def active(self):
        return self.get_queryset()

    def get_invited_by(self):
        return self.active()


class Invite(models.Model):

    invite = models.ForeignKey(User, related_name='invite', on_delete=models.CASCADE, verbose_name=_('Invite'))
    invited_by = models.ForeignKey(User, related_name='invite_by', on_delete=models.CASCADE, verbose_name=_('Invited By'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created At'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update At'))

    objects = InviteManager()

    def __str__(self):
        return '{}'.format(self.invite)

    class Meta:
        verbose_name = _('Invite')
        verbose_name_plural = _('Invites')

    def clean(self):
        # Don't allow complete action entries.
        if Invite.objects.filter(invite=self.invite).count() > 0:
            raise ValidationError({'invite': [_('This user invite before or invitation code is wrong.'), ]})

        if self.invited_by == self.invite:
            raise ValidationError({
                'invite': [_('Invite user can not be the same Invited user.'), ],
                'invited_by': [_('Invite user can not be the same Invited user.'), ]
                                   })

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Invite, self).save(*args, **kwargs)


def post_save_invite_receiver(sender, instance, created, *args, **kwargs):

    if created:
        from aparnik.packages.shops.vouchers.models import Voucher
        # پس از قبول دعوت مبلغ بالا به حساب دعوت شده واریز می شود در صورتی که عدد مخالف صفر باشد.
        gift = round(Setting.objects.get(key='INVITED_GIFT_BON_CREDITS').get_value() / Setting.objects.get(
            key='APARNIK_BON_VALUE').get_value())
        if gift > 0:
            # کسی که دعوت شده
            user_obj = instance.invite
            Voucher.objects.add_voucher_by_admin_command(user_obj, gift, description='کاربر %s شما را دعوت کرده و شما دعوت او را پذیرفته اید و تعداد %s بن برای شما اعمال شد.'%(instance.invited_by, gift))

        gift = round(Setting.objects.get(key='INVITER_GIFT_BON_CREDITS').get_value() / Setting.objects.get(
            key='APARNIK_BON_VALUE').get_value())
        # پس از قبول دعوت مبلغ بالا به حساب دعوت کننده واریز می شود در صورتی که عدد مخالف صفر باشد.
        if gift > 0:
            # کسی که دعوت کرده
            user_obj = instance.invited_by
            Voucher.objects.add_voucher_by_admin_command(user_obj, gift,
                                                         description='کاربر %s دعوت شما را پذیرفت و تعداد %s بن برای شما اعمال شد.' % (
                                                         instance.invite, gift))


post_save.connect(post_save_invite_receiver, sender=Invite)



# Generated by Django 2.2 on 2019-06-26 15:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0011_auto_20190626_1555'),
    ]

    operations = [
        migrations.RenameField(
            model_name='review',
            old_name='user_obj_2',
            new_name='user_obj',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-02 15:29


from django.db import migrations


def negative_default_value(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    User = apps.get_model('aparnik_users', 'User')
    User.objects.filter(limit_device_login=0).update(limit_device_login=-1)
    User.objects.filter(co_sale_percentage_value=0).update(co_sale_percentage_value=-1)


def zero_default_value(apps, schema_editor):
    User = apps.get_model('aparnik_users', 'User')
    User.objects.filter(limit_device_login=-1).update(limit_device_login=0)
    User.objects.filter(co_sale_percentage_value=-1).update(co_sale_percentage_value=0)


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0010_auto_20181202_1528'),
    ]

    operations = [
        migrations.RunPython(negative_default_value, reverse_code=zero_default_value),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-14 15:37


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0009_basemodel_sort'),
    ]

    operations = [
        migrations.AddField(
            model_name='basemodel',
            name='visit_count',
            field=models.IntegerField(default=0, verbose_name='Visit counter'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-19 17:44


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0016_product_is_draft'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='is_draft',
            field=models.BooleanField(default=False, verbose_name='Is draft'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from .models import Button


class ButtonAdmin(BaseModelAdmin):
    fields = ['title', 'icon', 'background_color', 'icon_color', 'title_color', 'model_obj']
    list_display = ['title', 'model_obj']
    search_fields = ['title']
    list_filter = []
    raw_id_fields = ['model_obj', 'icon']
    exclude = []
    dynamic_raw_id_fields = []        # exclude = ('password', 'iv',)
    inlines = []

    def __init__(self, *args, **kwargs):
        Klass = ButtonAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = Button


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import ButtonSegment

    class ButtonSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = ButtonSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = ButtonSegment


    admin.site.register(ButtonSegment, ButtonSegmentAdmin)


admin.site.register(Button, ButtonAdmin)



# -*- coding: utf-8 -*-


from django.shortcuts import get_object_or_404
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError


from .serializers import ProductSharingListSerializer
from ..models import ProductSharing


class ProductSharingListAPIView(ListAPIView):
    serializer_class = ProductSharingListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        queryset = ProductSharing.objects.share_this_user(user)
        if 'product_id' in self.request.parser_context['kwargs']:
            product_id = self.request.parser_context['kwargs']['product_id']
            queryset = ProductSharing.objects.share_this_user(user, product_id)
        return queryset


class ProductSharingSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, product_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        content = {
            'is_shared': False,
        }
        user = self.request.user
        from aparnik.packages.shops.products.models import Product
        from django.contrib.auth import get_user_model
        User = get_user_model()

        user_product_share_with = get_object_or_404(User, sys_id=self.request.data.get('user_product_share_with_id', None))

        product_obj = get_object_or_404(Product.objects.all(), id=product_id)

        try:
            obj = get_object_or_404(ProductSharing.objects.filter(product_obj=product_obj), user_obj=user,
                                    user_product_share_with_obj=user_product_share_with)
            obj.is_active = not obj.is_active
            obj.save()
            content['is_shared'] = obj.is_active
            status = HTTP_200_OK

        except Exception as e:
            try:

                obj = ProductSharing.objects.create(user_obj=user, product_obj=product_obj,
                                                    user_product_share_with_obj=user_product_share_with,
                                                    is_active=True)
                content['is_shared'] = obj.is_active
                status = HTTP_200_OK
            except Exception as e:
                raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)



# Generated by Django 2.2 on 2019-06-23 08:46

from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    key = ''
    try:
        key = 'USER_REGISTER_GIFT_BON_CREDITS'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='شارژ هدیه برای ثبت نام کاربر جدید',
            key=key,
            value='0',
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    try:
        key = 'USER_REGISTER_GIFT_BON_CREDITS'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0019_auto_20190525_1158'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from rest_framework.renderers import JSONRenderer
from dateutil.relativedelta import relativedelta

from aparnik.settings import aparnik_settings
from aparnik.contrib.notifications.models import Notification
from aparnik.packages.shops.subscriptions.models import SubscriptionOrder


json = JSONRenderer()
User = get_user_model()
UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class Command(BaseCommand):
    # Show this when the user types help
    help = "check subscription"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        # one week left
        one_week = now() + relativedelta(weeks=1)
        next_day = one_week + relativedelta(days=1)
        subscription_orders = SubscriptionOrder.objects.active().filter(expire_at__range=[one_week, next_day])
        self.send_notification(subscription_orders, 7)

        # one days left
        one_day = now()
        next_day = one_day + relativedelta(days=1)
        subscription_orders = SubscriptionOrder.objects.active().filter(expire_at__range=[one_day, next_day])
        self.send_notification(subscription_orders, 1)

        finished_time = now()

        print(('send Subscription %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def send_notification(self, queryset, days_remain):

        if queryset.count() == 0:
            return

        Notification.objects.send_notification(
            User.objects.active().filter(pk__in=queryset.values_list('order_obj__user', flat=True).distinct()),
            type=Notification.NOTIFICATION_INFO,
            title='اکانت وی آي پی شما %s روز دیگر منقضی می شود'%days_remain,
            description='لطفا برای استفاده از خدمات ویژه ما اکانت خود را تمدید نمایید.',
        )



from rest_framework import serializers

from aparnik.contrib.basemodels.api.admin.serializers import BaseModelAdminSerializer


from aparnik.contrib.users.api.serializers import UserSummaryListSerializer
from aparnik.packages.educations.teachers.models import Teacher


class TeacherAdminListSerializer(BaseModelAdminSerializer):

    user = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TeacherAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Teacher
        fields = BaseModelAdminSerializer.Meta.fields + [
            'user',
        ]

        read_only_fields = BaseModelAdminSerializer.Meta.read_only_fields + [
            'user',
        ]

    def get_user(self, obj):
        return UserSummaryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('province', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseEducation',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('is_published', models.BooleanField(default=False, verbose_name='Published')),
            ],
            options={
                'verbose_name': 'Base Education',
                'verbose_name_plural': 'Base Education',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.CreateModel(
            name='CityInstitute',
            fields=[
                ('baseeducation_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='educations.BaseEducation')),
            ],
            options={
                'verbose_name': 'CityInstitute',
                'verbose_name_plural': 'CityInstitutes',
            },
            bases=('educations.baseeducation',),
        ),
        migrations.CreateModel(
            name='Degree',
            fields=[
                ('baseeducation_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='educations.BaseEducation')),
                ('name', models.CharField(max_length=50, verbose_name='Title')),
            ],
            options={
                'verbose_name': 'Degree',
                'verbose_name_plural': 'Degrees',
            },
            bases=('educations.baseeducation',),
        ),
        migrations.CreateModel(
            name='Education',
            fields=[
                ('baseeducation_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='educations.BaseEducation')),
                ('receive_date', models.DateTimeField(null=True, verbose_name='Receive Date')),
                ('average', models.FloatField(max_length=3, null=True, verbose_name='Average')),
                ('description', models.TextField(blank=True, max_length=200, null=True, verbose_name='Descritpion')),
                ('degree_obj', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='educations.Degree', verbose_name='Degree')),
            ],
            options={
                'verbose_name': 'Education',
                'verbose_name_plural': 'Educations',
            },
            bases=('educations.baseeducation',),
        ),
        migrations.CreateModel(
            name='FieldSubject',
            fields=[
                ('baseeducation_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='educations.BaseEducation')),
                ('name', models.CharField(max_length=50, verbose_name='Title')),
                ('parent_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='educations.FieldSubject', verbose_name='Parent Field Subject')),
            ],
            options={
                'verbose_name': 'Field Subject',
                'verbose_name_plural': 'Field Subjects',
            },
            bases=('educations.baseeducation',),
        ),
        migrations.CreateModel(
            name='Institute',
            fields=[
                ('baseeducation_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='educations.BaseEducation')),
                ('name', models.CharField(max_length=40, verbose_name='Institute Name')),
                ('cities', models.ManyToManyField(through='educations.CityInstitute', to='province.City')),
            ],
            options={
                'verbose_name': 'Institute',
                'verbose_name_plural': 'Institutes',
            },
            bases=('educations.baseeducation',),
        ),
        migrations.AddField(
            model_name='education',
            name='field_subject_obj',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='educations.FieldSubject', verbose_name='Field Subject'),
        ),
        migrations.AddField(
            model_name='education',
            name='institute_obj',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='educations.Institute', verbose_name='Institute'),
        ),
        migrations.AddField(
            model_name='education',
            name='user_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='cityinstitute',
            name='Institute',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='educations.Institute', verbose_name='Institude'),
        ),
        migrations.AddField(
            model_name='cityinstitute',
            name='city',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='province.City', verbose_name='City'),
        ),
    ]



# Generated by Django 2.2 on 2019-06-15 14:30

import colorfield.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('buttons', '0006_auto_20190120_1724'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='button',
            options={'verbose_name': 'باتن', 'verbose_name_plural': 'باتن ها'},
        ),
        migrations.AlterModelOptions(
            name='buttonsegment',
            options={'verbose_name': 'بخش باتن', 'verbose_name_plural': 'بخش باتن ها'},
        ),
        migrations.AlterField(
            model_name='button',
            name='background_color',
            field=colorfield.fields.ColorField(default='#2672DF', max_length=18, verbose_name='رنگ پس زمینه'),
        ),
        migrations.AlterField(
            model_name='button',
            name='icon',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='آیکن'),
        ),
        migrations.AlterField(
            model_name='button',
            name='icon_color',
            field=colorfield.fields.ColorField(default='#2672DF', max_length=18, verbose_name='رنگ آیکن'),
        ),
        migrations.AlterField(
            model_name='button',
            name='model_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='button_model', to='basemodels.BaseModel', verbose_name='چه چیز را نمایش دهد؟'),
        ),
        migrations.AlterField(
            model_name='button',
            name='title',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='عنوان'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 11:31


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('books', '0002_auto_20181025_1543'),
    ]

    operations = [
        migrations.AlterField(
            model_name='book',
            name='translators_obj',
            field=models.ManyToManyField(blank=True, related_name='library_translators', to='books.WriterTranslator', verbose_name='Translators'),
        ),
    ]



# -*- coding: utf-8 -*-


from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer
from rest_framework.validators import UniqueTogetherValidator

from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer
from aparnik.packages.educations.educations.models import BaseEducation, Education, Degree, Institute, FieldSubject


# BASE EDUCATION
class BaseEducationListSerializer(BaseModelListSerializer):

    def __init__(self, *args, **kwargs):
        super(BaseEducationListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseEducation
        fields = BaseModelListSerializer.Meta.fields
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + []


class BaseEducationDetailsSerializer(BaseEducationListSerializer, BaseModelDetailSerializer):

    def __init__(self, *args, **kwargs):
        super(BaseEducationDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseEducation
        fields = BaseEducationListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields

        read_only_fields = BaseEducationListSerializer.Meta.read_only_fields + BaseModelDetailSerializer.Meta.read_only_fields + [
            'is_published']

    def get_model(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.model_obj, many=False, read_only=True, context=self.context).data


# EDUCATION
class EducationListSerializer(BaseEducationListSerializer):

    def __init__(self, *args, **kwargs):
        super(EducationListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Education
        fields = BaseEducationListSerializer.Meta.fields + [
            # 'user_obj',
        ]
        read_only_fields = BaseEducationListSerializer.Meta.read_only_fields + []


class EducationDetailsSerializers(EducationListSerializer,
                                  BaseEducationDetailsSerializer):
    institute_obj = serializers.SerializerMethodField()
    institute_obj_id = serializers.IntegerField(required=True, write_only=True)

    field_subject_obj = serializers.SerializerMethodField()
    field_subject_obj_id = serializers.IntegerField(required=True, write_only=True)

    degree_obj = serializers.SerializerMethodField()
    degree_obj_id = serializers.IntegerField(required=True, write_only=True)

    is_published = serializers.NullBooleanField()

    def __init__(self, *args, **kwargs):
        super(EducationDetailsSerializers, self).__init__(*args, **kwargs)

    class Meta:
        model = Education
        fields = EducationListSerializer.Meta.fields + BaseEducationDetailsSerializer.Meta.fields + [
            'receive_date',
            'average',
            'description',
            'institute_obj',
            'institute_obj_id',
            'field_subject_obj',
            'field_subject_obj_id',
            'degree_obj',
            'degree_obj_id',
            'is_published'
        ]

        validators = [
            UniqueTogetherValidator(
                queryset=Education.objects.all(),
                fields=('institute_obj_id', 'field_subject_obj_id', 'degree_obj_id')
            )
        ]

        read_only_fields = EducationListSerializer.Meta.read_only_fields + BaseEducationDetailsSerializer.Meta.read_only_fields + []

    def get_institute_obj(self, obj):
        return BaseEducationListPolymorphicSerializer(obj.institute_obj, many=False, read_only=True,
                                                      context=self.context).data

    def get_field_subject_obj(self, obj):
        return BaseEducationListPolymorphicSerializer(obj.field_subject_obj, many=False, read_only=True,
                                                      context=self.context).data

    def get_degree_obj(self, obj):
        return BaseEducationListPolymorphicSerializer(obj.degree_obj, many=False, read_only=True,
                                                      context=self.context).data

    def create(self, validated_data):
        institute_obj = Institute.objects.filter(id=validated_data.pop('institute_obj_id')).first()
        degree_obj = Degree.objects.filter(id=validated_data.pop('degree_obj_id')).first()
        field_subject_obj = FieldSubject.objects.filter(id=validated_data.pop('field_subject_obj_id')).first()

        education = Education.objects.create(institute_obj=institute_obj, degree_obj=degree_obj,
                                             field_subject_obj=field_subject_obj, **validated_data)

        return education

    def update(self, instance, validated_data):
        instance.institute_obj = Institute.objects.filter(id=validated_data.get('institute_obj_id', None)).get()
        instance.degree_obj = Degree.objects.filter(id=validated_data.get('degree_obj_id', None)).get()
        instance.field_subject_obj = FieldSubject.objects.filter(
            id=validated_data.get('field_subject_obj_id', None)).get()
        instance.receive_date = validated_data.get('receive_date', None)
        instance.average = validated_data.get('average', None)
        instance.description = validated_data.get('description', None)
        instance.is_published = validated_data.get('is_published', None)

        instance.save()

        return instance


# DEGREE
class DegreeListSerializer(BaseEducationListSerializer):
    def __init__(self, *args, **kwargs):
        super(DegreeListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Degree
        fields = BaseEducationListSerializer.Meta.fields + [
            'name',
        ]


class DegreeDetailsSerializer(DegreeListSerializer, BaseEducationDetailsSerializer):

    def __init__(self, *args, **kwargs):
        super(DegreeDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Degree
        fields = DegreeListSerializer.Meta.fields + BaseEducationDetailsSerializer.Meta.fields + [

        ]


# INSTITUDE
class InstituteListSerializer(BaseEducationListSerializer):

    def __init__(self, *args, **kwargs):
        super(InstituteListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Institute
        fields = BaseEducationListSerializer.Meta.fields + [
            'name'
        ]


class InstituteDetailsSerializer(InstituteListSerializer, BaseEducationDetailsSerializer):

    def __init__(self, *args, **kwargs):
        super(InstituteDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Institute
        fields = InstituteListSerializer.Meta.fields + BaseEducationDetailsSerializer.Meta.fields + [
            'name',
        ]


# FIELD SUBJECT
class FieldSubjectListSerializer(BaseEducationListSerializer):

    def __init__(self, *args, **kwargs):
        super(FieldSubjectListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = FieldSubject
        fields = BaseEducationListSerializer.Meta.fields + [
            'name',
        ]


class FieldSubjectDetailsSerializer(FieldSubjectListSerializer, BaseEducationDetailsSerializer):
    def __init__(self, *args, **kwargs):
        super(FieldSubjectDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = FieldSubject
        fields = FieldSubjectListSerializer.Meta.fields + BaseEducationDetailsSerializer.Meta.fields


class BaseEducationListPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            BaseEducation: BaseEducationListSerializer,
            Education: EducationListSerializer,
            Institute: InstituteListSerializer,
            FieldSubject: FieldSubjectListSerializer,
            Degree: DegreeListSerializer,
        }
        super(BaseEducationListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


class BaseEducationDetailsPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            BaseEducation: BaseEducationDetailsSerializer,
            Education: EducationDetailsSerializers,
            Institute: InstituteDetailsSerializer,
            FieldSubject: FieldSubjectDetailsSerializer,
            Degree: DegreeDetailsSerializer,
        }
        super(BaseEducationDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-


from django.utils.translation import ugettext as _
from django.shortcuts import get_object_or_404
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from .serializers import AddressDetailSerializer
from ..models import UserAddress


class AddressListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        return UserAddress.objects.this_user(user)


class AddressCreateAPIView(CreateAPIView):
    serializer_class = AddressDetailSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer):
        user = self.request.user
        serializer.save(user_obj=user)


class AddressSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, model_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'is_default': False,
        }

        try:
            address = get_object_or_404(UserAddress.objects.all(), id=model_id)
            content['is_default'] = address.set_is_default(user)
            status = HTTP_200_OK

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 16:02


from django.db import migrations, models
import django.db.models.deletion
import django.core.validators
from aparnik.utils import fields


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0001_initial'),
        ('aparnik_users', '0005_auto_20181025_1601'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='avatar',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='Avatar'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('review_average', models.FloatField(blank=True, null=True, verbose_name='Review Average')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_basemodels.basemodel_set+', to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ['-created_at'],
                'verbose_name': 'Base Model',
                'verbose_name_plural': 'Bases Models',
            },
        ),
    ]



# -*- coding: utf-8 -*-
import gettext as gettext_module
from threading import local
import os.path

__all__ = ['activate', 'deactivate', 'gettext', 'ngettext']

_TRANSLATIONS = {None: gettext_module.NullTranslations()}
_CURRENT = local()

_DEFAULT_LOCALE_PATH = os.path.join(os.path.dirname(__file__), 'locale')


def get_translation():
    try:
        return _TRANSLATIONS[_CURRENT.locale]
    except (AttributeError, KeyError):
        return _TRANSLATIONS[None]


def activate(locale, path=None):
    """Set 'locale' as current locale. Search for locale in directory 'path'
    @param locale: language name, eg 'en_GB'"""
    if path is None:
        path = _DEFAULT_LOCALE_PATH
    if locale not in _TRANSLATIONS:
        translation = gettext_module.translation('humanize', path, [locale])
        _TRANSLATIONS[locale] = translation
    _CURRENT.locale = locale
    return _TRANSLATIONS[locale]


def deactivate():
    _CURRENT.locale = None


def gettext(message):
    return get_translation().gettext(message)


def pgettext(msgctxt, message):
    """'Particular gettext' function.
    It works with 'msgctxt' .po modifiers and allow duplicate keys with
    different translations.
    Python 2 don't have support for this GNU gettext function, so we
    reimplement it. It works by joining msgctx and msgid by '4' byte."""
    key = msgctxt + '\x04' + message
    translation = get_translation().gettext(key)
    return message if translation == key else translation


def ngettext(message, plural, num):
    return get_translation().ngettext(message, plural, num)


def gettext_noop(message):
    """Example usage:
    CONSTANTS = [gettext_noop('first'), gettext_noop('second')]
    def num_name(n):
        return gettext(CONSTANTS[n])"""
    return message



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-21 18:56


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0005_auto_20190120_1724'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='uuid',
            field=models.UUIDField(blank=True, editable=False, null=True, verbose_name='Reference ID'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-20 17:24


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0004_orderitem_is_tax'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='orderitem',
            options={'ordering': ('created_at',), 'verbose_name': 'OrderItem', 'verbose_name_plural': 'OrderItems'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-01 14:21


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0004_filefield_title'),
        ('files', '0003_remove_file_is_preview'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='cover',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_cover', to='filefields.FileField', verbose_name='\\u062a\\u0635\\u0648\\u06cc\\u0631 \\u062c\\u0644\\u062f'),
        ),
    ]



from rest_framework import serializers

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer
from ..models import SocialNetwork
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer


class SocialNetworkDestailSerializer(BaseModelDetailSerializer):

    icon = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SocialNetworkDestailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SocialNetwork
        fields = BaseModelDetailSerializer.Meta.fields + [
            'link',
            'icon',
            'title',
            'android_app_shortcut',
            'ios_app_shortcut',
            'value',
        ]

    def get_icon(self, obj):
        return FileFieldListSerailizer(obj.icon, many=False, read_only=True, context=self.context).data


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import SocialNetworkSegment

    # Base Segment ListSegmentReview
    class SegmentSocialNetworkListSerializer(BaseSegmentListSerializer):

        class Meta:
            model = SocialNetworkSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentSocialNetworkDetailSerializer(BaseSegmentDetailSerializer, SegmentSocialNetworkListSerializer):

        class Meta:
            model = SocialNetworkSegment
            fields = SegmentSocialNetworkListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentSocialNetworkListSerializer.Meta.read_only_fields + []




from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer

from ..models import ShortBlog


class ShortBlogListSerializer(ModelSerializer):
    datetime = serializers.SerializerMethodField()
    # category = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ShortBlogListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ShortBlog
        fields = [
            'id',
            'title',
            'content',
            'datetime',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_datetime(self, obj):
        return obj.update_at

    # def get_category(self, obj):
    #     return CategoryListSerializer(obj.category, many=True, read_only=True, context=self.context).data


class ShortBlogDetailSerializer(ShortBlogListSerializer):

    def __init__(self, *args, **kwargs):
        super(ShortBlogDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ShortBlog
        fields = ShortBlogListSerializer.Meta.fields + [
        ]




# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-29 17:28


import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vouchers', '0003_auto_20190129_1620'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='expire_at',
            field=models.DateTimeField(default=datetime.datetime(2219, 1, 29, 17, 28, 5, 432761), verbose_name='Expire at'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.socials.models import SocialNetwork
from aparnik.utils.fields import PhoneField


# Create your models here.
class SupportManager(BaseModelManager):

    def get_queryset(self):
        return super(SupportManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_active=True)


class Support(BaseModel):
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    online = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Online'))
    offline = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Offline'))
    phone = PhoneField(blank=True, null=True, mobile=False, verbose_name=_('Phone'))
    online_days = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Online Days'))

    socials = models.ManyToManyField(SocialNetwork, verbose_name=_('Socials'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = SupportManager()

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _('Support')
        verbose_name_plural = _('Supports')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-04 10:50


from django.db import migrations
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0006_coursefile_seconds'),
    ]

    def convert_to_seconds(apps, schema_editor):

        Object = apps.get_model('courses', 'CourseFile')
        for obj in Object.objects.all():
            obj.seconds = datetime.timedelta(hours=obj.time.hour, minutes=obj.time.minute,
                                              seconds=obj.time.second).total_seconds()
            obj.save()

    operations = [
        migrations.RunPython(convert_to_seconds, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-17 20:56


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0013_product_currency'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='is_tax',
            field=models.BooleanField(default=False, verbose_name='Is tax?'),
        ),
    ]



# -*- coding: utf-8 -*-
'''
Module: num2word_EU.py
Requires: num2word_base.py
Version: 1.1

Author:
   Taro Ogawa (tso@users.sourceforge.org)
   
Copyright:
    Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.

Licence:
    This module is distributed under the Lesser General Public Licence.
    http://www.opensource.org/licenses/lgpl-license.php

Data from:
    http://www.uni-bonn.de/~manfear/large.php

History:
    1.1: add to_currency()
'''
from .num2word_base import Num2Word_Base


class Num2Word_EU(Num2Word_Base):
    def set_high_numwords(self, high):
        max = 3 + 6 * len(high)

        for word, n in zip(high, list(range(max, 3, -6))):
            self.cards[10 ** n] = word + "یارد"
            self.cards[10 ** (n - 3)] = word + "یلیون"

    def set_high_numwords_str(self, high):
        max = 3 + 6 * len(high)

        for word, n in zip(high, list(range(max, 3, -6))):
            self.cards_str[10 ** n] = word + "یارد"
            self.cards_str[10 ** (n - 3)] = word + "یلیون"

    def base_setup(self):
        lows = ["نون", "اکت", "سپت", "سکست", "کوینت", "کوادر", "تر", "", ""]
        units = ["", "un", "duo", "tre", "quattuor", "quin", "sex", "sept",
                 "octo", "novem"]
        tens = ["dec", "vigint", "trigint", "quadragint", "quinquagint",
                "sexagint", "septuagint", "octogint", "nonagint"]
        self.high_numwords = ["cent"] + self.gen_high_numwords(units, tens, lows)
        self.high_numwords_str = ["cent"] + self.gen_high_numwords(units, tens, lows)

    def to_currency(self, val, longval=True, jointxt=""):
        return self.to_splitnum(val, hightxt="Euro/s", lowtxt="Euro cent/s",
                                jointxt=jointxt, longval=longval)



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.packages.shops.products.admin import ProductAdmin
from .models import File
# from questionsanswer.models import QA, Like


class FileAdmin(ProductAdmin):

    fields = ['description', 'file_obj', 'banner', 'cover', 'publish_date']
    list_display = []
    list_filter = []
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []
    inlines = []
    raw_id_fields = ['file_obj', 'cover', 'banner']

    def __init__(self, *args, **kwargs):
        Klass = FileAdmin
        Klass_parent = ProductAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = File


admin.site.register(File, FileAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 16:11


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0004_auto_20181213_1607'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='basesegment',
            name='model_obj',
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf.urls import url
from .views import ProgressSetAPIView

app_name = 'progresses'

urlpatterns = [
    # url(r'^$', ProgressListAPIView.as_view(), name='list'),
    url(r'^(?P<file_id>[\w-]+)/set/$', ProgressSetAPIView.as_view(), name='set'),
    # url(r'^(?P<progress_id>[\w-]+)/update', ProgressUpdateAPIView.as_view(), name='update'),
]



from django.conf.urls import url, include

from .views import CourseFileListAPIView, CourseFileCreateAPIView

app_name = 'courses'

urlpatterns = [
    url(r'^$', CourseFileListAPIView.as_view(), name='list'),
    url(r'^create/$', CourseFileCreateAPIView.as_view(), name='create'),
    # url(r'^(?P<file_id>\d+)/download-request/$', FileDownloadRequestAPIView.as_view(), name='download-request'),
    # url(r'^preview/$', FileListAPIView.as_view(), name='preview-list'),

    # url(r'^(?P<file_id>\d+)/$', FileListAPIView.as_view(), name='detail'),
    # url(r'^(?P<id>\d+)/file/', include('courses.api.urls-files', namespace='file')),
    # url(r'^(?P<id>\d+)/order/$', CourseOrderAPIView.as_view(), name='order'),
]


from rest_framework.renderers import JSONRenderer

class JSONRenderer(JSONRenderer):

    def render(self, data, accepted_media_type=None, renderer_context=None):
        # data = {
        #     "msg": "",
        #     "data": data,
        #     "success": True
        # }
        return super(JSONRenderer, self).render(data, accepted_media_type, renderer_context)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-28 11:07


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0010_order_postal_cost_value'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='is_sync_with_websites',
            field=models.BooleanField(default=False, verbose_name='Is sync with websites'),
        ),
        migrations.AlterField(
            model_name='order',
            name='status',
            field=models.CharField(choices=[('w', 'Waiting'), ('pa', 'Paid'), ('paw', 'Paid by website'), ('c', 'Cancel'), ('co', 'Complete'), ('d', 'Disputed'), ('ch', 'Challenged')], default='w', max_length=5, verbose_name='Status'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-28 21:16


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sliders', '0002_slidersegment'),
        ('teachers', '0002_auto_20181026_1301'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='teacher',
            name='slider_obj',
        ),
        migrations.AddField(
            model_name='teacher',
            name='slider_segment_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sliders.SliderSegment', verbose_name='Slider Semgent'),
        ),
    ]



from rest_framework.relations import PKOnlyObject
from collections import OrderedDict
from rest_framework import serializers
from rest_framework.fields import SkipField

from aparnik.settings import aparnik_settings


class ModelSerializer(serializers.ModelSerializer):

    def to_representation(self, instance):
        """Object instance -> Dict of primitive datatypes."""
        ret = OrderedDict()
        fields = self._readable_fields

        for field in fields:
            try:
                attribute = field.get_attribute(instance)
            except SkipField:
                continue

            # We skip `to_representation` for `None` values so that fields do
            # not have to explicitly deal with that case.
            #
            # For related fields with `use_pk_only_optimization` we need to
            # resolve the pk value.
            check_for_none = attribute.pk if isinstance(attribute, PKOnlyObject) else attribute
            if check_for_none is None:
                value = None
            else:
                value = field.to_representation(attribute)

            if aparnik_settings.API_PRODUCT_MODE:
                ret[field.field_name] = {
                    'value': value,
                    # You can find more field attributes here
                    # https://github.com/encode/django-rest-framework/blob/master/rest_framework/fields.py#L324
                    'verbose_name': field.label,
                    'read_only': field.read_only,
                    'placeholder': field.label,
                    'type': type(value).__name__,
                    # 'write_only': field.write_only,
                    'help_text': field.help_text,
                }
            else:
                ret[field.field_name] = value

        return ret



# from rest_framework import serializers
# from aparnik.api.serializers import ModelSerializer
# from coupons.models import Coupon
#
# class CouponSummarySerializer(ModelSerializer):
#
#     discount = serializers.SerializerMethodField()
#
#     class Meta:
#         model = Coupon
#         fields = [
#             'discount',
#         ]
#
#     def get_url(self, obj):
#         return self.context['request'].build_absolute_uri(obj.get_api_uri())
#
#     def get_total_cost(self, obj):
#         return obj.get_total_cost()
#
#     def get_total_cost_order(self, obj):
#         return obj.get_total_cost_order()
#
#     def get_url_request_pay(self, obj):
#         return self.context['request'].build_absolute_uri(obj.get_api_request_pay_uri())
#
#     def get_title(self, obj):
#         item = obj.items.first()
#         if item:
#             return item.product_obj.title
#         return ""
#
#     def get_discount(self, obj):
#         return obj.get_discount()


from django.conf.urls import url, include

from .views import BankAccountListAPIView, BankAccountSetAPIView, BankAccountUpdateAPIView, BankAccountCreateAPIView, BankNameListAPIView

app_name = 'bankaccounts'

urlpatterns = [
    url(r'^$', BankAccountListAPIView.as_view(), name='list'),
    url(r'^bankname/$', BankNameListAPIView.as_view(), name='bankname-list'),
    url(r'^(?P<model_id>\d+)/set/$', BankAccountSetAPIView.as_view(), name='set'),
    url(r'^(?P<model_id>\d+)/update/$', BankAccountUpdateAPIView.as_view(), name='update'),
    url(r'^create/$', BankAccountCreateAPIView.as_view(), name='create'),
]


# -*- coding: utf-8 -*-


from django.contrib import admin
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from aparnik.contrib.users.admin import get_user_search_fields

from .models import CoSale, CoSalePayment


# class CoSaleHistoryInline(admin.TabularInline):
#     exclude = ['visit_count', 'review_average', 'sort', 'tags', 'update_needed', 'is_show_only_for_super_user']
#     extra = 0
#     model = CoSaleHistory
#     fk_name = 'cosale_obj'


class CoSaleAdmin(BaseModelAdmin):
    fields = ['user_bought_obj', 'user_co_sale_obj', 'order_obj', 'is_active', 'price']
    list_display = ['user_bought_obj', 'user_co_sale_obj', 'order_obj', 'is_active', 'price']
    # inlines = [CoSaleHistoryInline]
    inlines = []
    list_filter = []
    search_fields = ['user_bought_obj__username', 'user_co_sale_obj__username', 'order_obj__id']
    exclude = []
    raw_id_fields = ['user_bought_obj', 'user_co_sale_obj', 'order_obj']
    dynamic_raw_id_fields = []
    readonly_fields = []

    def __init__(self, *args, **kwargs):
        Klass = CoSaleAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


class CoSalePaymentAdmin(BaseModelAdmin):
    fields = ['price', 'user_bank_account_obj', 'tracking_number', 'status']
    list_display = ['price', 'user_bank_account_obj', 'tracking_number', 'status']
    inlines = []
    list_filter = ['status']
    search_fields = get_user_search_fields('user_bank_account_obj__user_obj') + []
    exclude = []
    raw_id_fields = ['user_bank_account_obj']
    dynamic_raw_id_fields = []
    readonly_fields = []

    def __init__(self, *args, **kwargs):
        Klass = CoSalePaymentAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


admin.site.register(CoSale, CoSaleAdmin)
admin.site.register(CoSalePayment, CoSalePaymentAdmin)



from django.db.models import Sum, Q
from django.db.models.functions import Coalesce


from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.packages.shops.products.api.admin.views import ProductSortAdminAPIView
from aparnik.packages.educations.progresses.models import ProgressSummary
from .serializers import Course, CourseAdminListSerializer


class CourseSortAdminAPIView(ProductSortAdminAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(CourseSortAdminAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }
        self.command_dict.update(ProgressSummary.sort_progress(prefix='progress_summaries'))
        self.command_dict.update(command_dict)


class CourseAdminListAPIView(AdminListAPIView):
    serializer_class = CourseAdminListSerializer
    queryset = Course.objects.all()

    def get_sort_api(self, request):
        return CourseSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager


User = get_user_model()


# TODO: send notification when change available in course or etc.
# Product Sharing
class NotifyMeManager(BaseModelManager):
    def get_queryset(self):
        return super(NotifyMeManager, self).get_queryset()

    def active(self):
        return super(NotifyMeManager, self).active().filter(is_active=True)

    def this_user(self, user):
        if not user.is_authenticated:
            return NotifyMe.objects.none()
        return self.active().filter(user_obj=user)


class NotifyMe(BaseModel):

    user_obj = models.ForeignKey(User, related_name='notifyme_user', on_delete=models.CASCADE, verbose_name=_('User'))
    model_obj = models.ForeignKey(BaseModel, related_name='notifyme_model', on_delete=models.CASCADE, verbose_name=_('Model'))
    is_active = models.BooleanField(default=False, verbose_name=_('Is Active'))

    objects = NotifyMeManager()

    class Meta:
        verbose_name = _('Notify Me')
        verbose_name_plural = _('Notifies Me')

    def __str__(self):
        return str(self.user_obj)

    def save(self, *args, **kwargs):
        self.full_clean()
        if not self.id:
            self.update_needed = True
        # TODO: send Notif
        return super(NotifyMe, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('files', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Book',
            fields=[
                ('file_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='files.File')),
                ('published_date', models.DateTimeField(verbose_name='Published Date')),
            ],
            options={
                'verbose_name': 'Book',
                'verbose_name_plural': 'Books',
            },
            bases=('files.file',),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-22 21:08


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0008_auto_20190121_2101'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderitem',
            name='description',
            field=models.TextField(blank=True, default='', null=True, verbose_name='Description'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 17:10


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    CourseFile = apps.get_model('courses', 'CourseFile')
    for course_file in CourseFile.objects.all():
        course_file.file_obj.seconds = course_file.seconds
        course_file.file_obj.save()


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    # FileField = apps.get_model('filefields', 'FileField')
    CourseFile = apps.get_model('courses', 'CourseFile')
    for course_file in CourseFile.objects.all():
        course_file.seconds = course_file.file_obj.seconds
        course_file.save()


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0007_filefield_seconds'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-26 12:10


from django.db import migrations
from aparnik.contrib.users.models import DeviceType


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    DeviceLogin = apps.get_model('aparnik_users', 'devicelogin')
    for device in DeviceLogin.objects.all():
        device.device_type = DeviceType.ANDROID if device.device_type_temp == 'a' else DeviceType.IOS
        device.save()


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    DeviceLogin = apps.get_model('aparnik_users', 'devicelogin')
    for device in DeviceLogin.objects.all():
        device.device_type_temp = 'a' if device.device_type == DeviceType.ANDROID else 'i'
        device.save()


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0025_auto_20190526_1210'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.html import format_html
from django.urls import reverse
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import Order, OrderItem


# Rcegister your models here.
class OrderItemInline(BaseModelTabularInline):
    model = OrderItem
    raw_id_fields = ['product_obj']
    fk_name = 'order_obj'
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = OrderItemInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


# class OrderItemAdmin(BaseModelAdmin):
#     list_display = ['order_obj', 'product', 'price', 'quantity']
#     raw_id_fields = ['order_obj']
#     search_fields = ['order_obj__id', 'product_obj__title']
#     fields = ['__all__']
#     inlines = [OrderItemInline]
#     list_filter = ['status', 'created_at']
#     exclude = []
#     dynamic_raw_id_fields = []
#
#     def __init__(self, *args, **kwargs):
#         Klass = OrderItemAdmin
#         Klass_parent = BaseModelAdmin
#
#         super(Klass, self).__init__(*args, **kwargs)
#         self.fields = Klass_parent.fields + self.fields
#         self.list_display = Klass_parent.list_display + self.list_display
#         self.list_filter = Klass_parent.list_filter + self.list_filter
#         self.search_fields = Klass_parent.search_fields + self.search_fields
#         self.exclude = Klass_parent.exclude + self.exclude
#         self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
#         self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
#
#     class Meta:
#         OrderItem

def invoice_link(obj):
    return format_html(
        '<a href="{0}" target="blank" class="button">Invoice</a>',
        obj.get_pay_uri()
    )


invoice_link.short_description = "Invoice"


class OrderAdmin(BaseModelAdmin):
    # list_display = ['id', 'first_name', 'last_name', 'email',
    #                 'address', 'postal_code', 'city', 'paid',
    #                 'created', 'updated']
    # list_filter = ['paid', 'created', 'updated']
    fields = ['user', 'status', 'address_obj']
    list_display = ['user', 'coupon', 'uuid', invoice_link, 'status', 'is_sync_with_websites', get_update_at]
    inlines = [OrderItemInline]
    list_filter = ['status', 'created_at', 'is_sync_with_websites']
    search_fields = get_user_search_fields('user') + ['coupon__code', 'uuid']
    exclude = []
    raw_id_fields = ['user', 'coupon', 'address_obj']
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = OrderAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


admin.site.register(Order, OrderAdmin)
# admin.site.register(OrderItem, OrderItemAdmin)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('segments', '0001_initial'),
        ('filefields', '0001_initial'),
        ('teachers', '0001_initial'),
        ('categories', '0001_initial'),
        ('courses', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CourseSegment',
            fields=[
                ('basesegment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='segments.BaseSegment')),
            ],
            options={
                'verbose_name': 'Course Segment',
                'verbose_name_plural': 'Courses Segments',
            },
            bases=('segments.basesegment',),
        ),
        migrations.CreateModel(
            name='CourseSummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('total_time_seconds', models.CharField(default='0', max_length=255, verbose_name='\\u0632\\u0645\\u0627\\u0646')),
                ('file_count', models.PositiveIntegerField(default=0, verbose_name='File Count')),
                ('file_count_preview', models.PositiveIntegerField(default=0, verbose_name='File Count Preview')),
                ('type', models.CharField(choices=[('M', 'Movie'), ('V', 'Voice'), ('P', 'PDF'), ('I', '\\u062a\\u0635\\u0648\\u06cc\\u0631')], max_length=1, verbose_name='Type')),
            ],
            options={
                'verbose_name': 'Course Summary',
                'verbose_name_plural': 'Courses Summary',
            },
        ),
        migrations.CreateModel(
            name='Course',
            fields=[
                ('basecourse_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='courses.BaseCourse')),
                ('sort', models.IntegerField(default=0, verbose_name='Sort')),
                ('is_free', models.BooleanField(default=False, verbose_name='Is Free')),
                ('publish_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Publish Date')),
                ('publish_month', models.CharField(blank=True, choices=[('1', 'Farvardin'), ('2', 'Ordibehesht'), ('3', 'Khordad'), ('4', 'Tir'), ('5', 'Mordad'), ('6', 'Shahrivar'), ('7', 'Mehr'), ('8', 'Aban'), ('9', 'Azar'), ('10', 'Dey'), ('11', 'Bahman'), ('12', 'Esfand')], max_length=2, null=True, verbose_name='Month')),
                ('publish_week', models.CharField(blank=True, choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4')], max_length=1, null=True, verbose_name='Week')),
                ('publish_day', models.CharField(blank=True, choices=[('1', '\\u0634\\u0646\\u0628\\u0647'), ('2', '\\u06cc\\u06a9\\u0634\\u0646\\u0628\\u0647'), ('3', '\\u062f\\u0648\\u0634\\u0646\\u0628\\u0647'), ('4', '\\u0633\\u0647 \\u0634\\u0646\\u0628\\u0647'), ('5', '\\u0686\\u0647\\u0627\\u0631\\u0634\\u0646\\u0628\\u0647'), ('6', '\\u067e\\u0646\\u062c\\u0634\\u0646\\u0628\\u0647'), ('7', '\\u062c\\u0645\\u0639\\u0647')], max_length=1, null=True, verbose_name='DAY')),
                ('banner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_banner', to='filefields.FileField', verbose_name='Banner Image')),
                ('category_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='categories.Category', verbose_name='Category')),
                ('cover', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_cover', to='filefields.FileField', verbose_name='Cover Image')),
                ('dependency_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dependency', to='courses.Course', verbose_name='Dependency')),
                ('parent_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='Parent')),
                ('teacher_obj', models.ManyToManyField(blank=True, null=True, to='teachers.Teacher', verbose_name='Teachers')),
            ],
            options={
                'verbose_name': 'Course',
                'verbose_name_plural': 'Courses',
            },
            bases=('courses.basecourse',),
        ),
        migrations.AddField(
            model_name='coursesummary',
            name='course',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='total_times', to='courses.BaseCourse', verbose_name='Course'),
        ),
        migrations.AddField(
            model_name='coursefile',
            name='course_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='Course'),
        ),
    ]



from django.db.models import Count, Q


from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.basemodels.api.admin.views import BaseModelSortAdminAPIView
from aparnik.contrib.bookmarks.models import Bookmark
from aparnik.packages.shops.orders.models import Order
from aparnik.packages.shops.vouchers.models import Voucher
from aparnik.packages.shops.coupons.models import Coupon

from .serializers import Product, ProductAdminListSerializer


class ProductSortAdminAPIView(BaseModelSortAdminAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(ProductSortAdminAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }
        command_dict.update(
            Coupon.sort_redeem(prefix='orderitem_set__order_obj__coupon__membership', return_key='coupons_product'))
        command_dict.update(Voucher.sort_voucher(prefix='orderitem_set__voucher_item_spent', return_key='voucher_product'))
        command_dict.update(Bookmark.sort_bookmark(prefix='bookmark_obj'))
        command_dict.update(Order.sort_buy_waiting(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_wallet(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_bank(prefix='orderitem_set__order_obj'))

        self.command_dict.update(command_dict)


class ProductAdminListAPIView(AdminListAPIView):
    serializer_class = ProductAdminListSerializer
    queryset = Product.objects.all()

    def get_sort_api(self, request):
        return ProductSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 11:42


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0005_auto_20190501_1142'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filefield',
            name='is_lock',
            field=models.BooleanField(default=True, verbose_name='Is lock?'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('shortblogs', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TermsAndConditions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('short_blogs', models.ManyToManyField(to='shortblogs.ShortBlog', verbose_name='Content Segment')),
            ],
            options={
                'verbose_name': 'Terms And Conditions',
                'verbose_name_plural': 'Terms And Conditions',
            },
        ),
    ]



# -*- coding: utf-8 -*-
'''
Module: num2word_base.py
Version: 1.0

Author:
   Taro Ogawa (tso@users.sourceforge.org)
   
Copyright:
    Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.

Licence:
    This module is distributed under the Lesser General Public Licence.
    http://www.opensource.org/licenses/lgpl-license.php

History:
    1.1: add to_splitnum() and inflect()
         add to_year() and to_currency() stubs
'''


from .orderedmapping import OrderedMapping


class Num2Word_Base(object):
    def __init__(self):
        self.cards = OrderedMapping()
        self.cards_str = OrderedMapping()
        self.is_title = False
        self.precision = 2
        self.exclude_title = []
        self.negword = "(-) "
        self.pointword = "(.)"
        self.errmsg_nonnum = "type(%s) not in [long, int, float]"
        self.errmsg_floatord = "Cannot treat float %s as ordinal."
        self.errmsg_negord = "Cannot treat negative num %s as ordinal."
        self.errmsg_toobig = "abs(%s) must be less than %s."

        self.base_setup()
        self.setup()
        self.set_numwords()
        self.set_numwords_str()

        self.MAXVAL = 1000 * self.cards.order[0]

    def set_numwords(self):
        self.set_high_numwords(self.high_numwords)
        self.set_mid_numwords(self.mid_numwords)
        self.set_low_numwords(self.low_numwords)

    def set_numwords_str(self):
        self.set_high_numwords_str(self.high_numwords_str)
        self.set_mid_numwords_str(self.mid_numwords_str)
        self.set_low_numwords_str(self.low_numwords_str)

    def gen_high_numwords(self, units, tens, lows):
        out = [u + t for t in tens for u in units]
        out.reverse()
        return out + lows

    def gen_high_numwords_str(self, units, tens, lows):
        out = [u + t for t in tens for u in units]
        out.reverse()
        return out + lows

    def set_mid_numwords(self, mid):
        for key, val in mid:
            self.cards[key] = val

    def set_mid_numwords_str(self, mid):
        for key, val in mid:
            self.cards_str[key] = val

    def set_low_numwords(self, numwords):
        for word, n in zip(numwords, list(range(len(numwords) - 1, -1, -1))):
            self.cards[n] = word

    def set_low_numwords_str(self, numwords):
        for word, n in zip(numwords, list(range(len(numwords) - 1, -1, -1))):
            self.cards_str[n] = word

    def splitnum(self, value):
        for elem in self.cards:
            if elem > value:
                continue

            out = []
            if value == 0:
                div, mod = 1, 0
            else:
                div, mod = divmod(value, elem)
            # For values of x1000 append its name
            if div == 1 and value >= 1000:
                out.append((str(1), 1))

            else:
                if div == value:  # The system tallies, eg Roman Numerals
                    return [(div * self.cards[elem], div * elem)]
                if (div > 1000 and div % 10 != 0) or (div > 1000):
                    out.append(self.splitnum(div))
                elif (div != 0 and div != 1):
                    out.append((str(div), div))

            # For values of x1000 append its name
            if round(elem / 1000) != 0:
                out.append((self.cards[elem], elem))
            else:
                out.append((str(elem), elem))

            if (mod >= 1000 and mod % 10 != 0) or (mod >= 1000):
                out.append(self.splitnum(mod))
            elif (mod != 0 and mod % 10 == 0):
                out.append((str(mod), mod))
            elif (mod != 0):
                out.append((str(mod), mod))

            return out

    def splitnum_str(self, value):
        for elem in self.cards_str:
            if elem > value:
                continue

            out = []
            if value == 0:
                div, mod = 1, 0
            else:
                div, mod = divmod(value, elem)

            if div == 1:  # TODO: This condition needs a better replacement
                if (mod != 0 and value == elem) or (mod >= 1000 and value == elem) or (
                        value == elem and value >= 1000 and div > 0):
                    out.append((self.cards_str[1], 1))

            else:
                if div == value:  # The system tallies, eg Roman Numerals
                    return [(div * self.cards_str[elem], div * elem)]
                if (div > 20 and div % 10 != 0) or (div > 20):
                    out.append(self.splitnum_str(div))
                elif (div != 0):
                    out.append((self.cards_str[div], div))

            out.append((self.cards_str[elem], elem))

            if (mod > 20 and mod % 10 != 0) or (mod > 20):
                out.append(self.splitnum_str(mod))
            elif (mod != 0):
                out.append((self.cards_str[mod], mod))

            return out

    def to_cardinal(self, value):
        try:
            assert int(value) == value
        except (ValueError, TypeError, AssertionError):
            return self.to_cardinal_float(value)

        self.verify_num(value)

        out = ""
        if value < 0:
            value = abs(value)
            out = self.negword

        if value >= self.MAXVAL:
            raise OverflowError(self.errmsg_toobig % (value, self.MAXVAL))

        val = self.splitnum(value)
        words, num = self.clean(val)
        return self.title(out + words)

    def to_cardinal_str(self, value):
        try:
            assert int(value) == value
        except (ValueError, TypeError, AssertionError):
            return self.to_cardinal_float(value)

        self.verify_num(value)

        out = ""
        if value < 0:
            value = abs(value)
            out = self.negword

        if value >= self.MAXVAL:
            raise OverflowError(self.errmsg_toobig % (value, self.MAXVAL))

        val = self.splitnum_str(value)
        words, num = self.clean_str(val)
        return self.title_str(out + words)

    def to_cardinal_float(self, value):
        try:
            float(value) == value
        except (ValueError, TypeError, AssertionError):
            raise TypeError(self.errmsg_nonnum % value)

        pre = int(value)
        post = abs(value - pre)

        out = [self.to_cardinal(pre)]
        if self.precision:
            out.append(self.title(self.pointword))

        for i in range(self.precision):
            post *= 10
            curr = int(post)
            out.append(str(self.to_cardinal(curr)))
            post -= curr

        return " ".join(out)

    def merge(self, curr, next):
        raise NotImplementedError

    def merge_str(self, curr, next):
        raise NotImplementedError

    def clean(self, val):
        out = val
        while len(val) != 1:
            out = []
            curr, next = val[:2]
            if isinstance(curr, tuple) and isinstance(next, tuple):
                out.append(self.merge(curr, next))
                if val[2:]:
                    out.append(val[2:])
            else:
                for elem in val:
                    if isinstance(elem, list):
                        if len(elem) == 1:
                            out.append(elem[0])
                        else:
                            out.append(self.clean(elem))
                    else:
                        out.append(elem)
            val = out
        return out[0]

    def clean_str(self, val):
        out = val
        while len(val) != 1:
            out = []
            curr, next = val[:2]
            if isinstance(curr, tuple) and isinstance(next, tuple):
                out.append(self.merge_str(curr, next))
                if val[2:]:
                    out.append(val[2:])
            else:
                for elem in val:
                    if isinstance(elem, list):
                        if len(elem) == 1:
                            out.append(elem[0])
                        else:
                            out.append(self.clean_str(elem))
                    else:
                        out.append(elem)
            val = out
        return out[0]

    def title(self, value):
        if self.is_title:
            out = []
            value = value.split()
            for word in value:
                if word in self.exclude_title:
                    out.append(word)
                else:
                    out.append(word[0].upper() + word[1:])
            value = " ".join(out)
        return value

    def title_str(self, value):
        if self.is_title:
            out = []
            value = value.split()
            for word in value:
                if word in self.exclude_title:
                    out.append(word)
                else:
                    out.append(word[0].upper() + word[1:])
            value = " ".join(out)
        return value

    def verify_ordinal(self, value):
        if not value == int(value):
            raise TypeError(self.errmsg_floatord % (value))
        if not abs(value) == value:
            raise TypeError(self.errmsg_negord % (value))

    def verify_num(self, value):
        return 1

    def set_wordnums(self):
        pass

    def to_ordinal(self, value):
        return self.to_cardinal(value)

    def to_ordinal_num(self, value):
        return value

    # Trivial version
    def inflect(self, value, text):
        text = text.split("/")
        if value == 1:
            return text[0]
        return "".join(text)

    def to_splitnum(self, val, hightxt="", lowtxt="", jointxt="",
                    divisor=100, longval=True):
        out = []
        try:
            high, low = val
        except TypeError:
            high, low = divmod(val, divisor)
        if high:
            hightxt = self.title(self.inflect(high, hightxt))
            out.append(self.to_cardinal(high))
            if low:
                if longval:
                    if hightxt:
                        out.append(hightxt)
                    if jointxt:
                        out.append(self.title(jointxt))
            elif hightxt:
                out.append(hightxt)
        if low:
            out.append(self.to_cardinal(low))
            if lowtxt and longval:
                out.append(self.title(self.inflect(low, lowtxt)))
        return " ".join(out)

    def to_year(self, value, **kwargs):
        return self.to_cardinal(value)

    def to_currency(self, value, **kwargs):
        return self.to_cardinal(value)

    def base_setup(self):
        pass

    def setup(self):
        pass

    def test(self, value):
        try:
            _card = self.to_cardinal_str(value)
        except:
            _card = "invalid"
        try:
            _ord = self.to_ordinal(value)
        except:
            _ord = "invalid"
        try:
            _ordnum = self.to_ordinal_num(value)
        except:
            _ordnum = "invalid"

        print(("For %s, card is %s;\n\tord is %s; and\n\tordnum is %s." %
               (value, _card, _ord, _ordnum)))



# Generated by Django 2.2 on 2019-06-26 15:55

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questionanswers', '0010_auto_20190626_1549'),
        ('reviews', '0010_auto_20190626_1549'),
    ]

    operations = [
        migrations.AlterField(
            model_name='qa',
            name='user_obj_2',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='کاربر'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-10 14:42


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0003_remove_category_pages'),
        ('news', '0002_auto_20181209_2322'),
    ]

    operations = [
        migrations.AddField(
            model_name='news',
            name='categories',
            field=models.ManyToManyField(to='categories.Category', verbose_name='\\u062f\\u0633\\u062a\\u0647 \\u0628\\u0646\\u062f\\u06cc \\u0647\\u0627'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0007_auto_20181025_1602'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='devicelogin',
            options={'verbose_name': 'Device Login', 'verbose_name_plural': 'Devices Login'},
        ),
    ]



from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.utils.utils import is_app_installed
from aparnik.api.serializers import ModelSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from ..models import *


# Slider Details Slider
class SliderDetailsSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    image = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SliderDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Slider
        fields = [
            'id',
            'url',
            'image',
            'name',
        ]

    def get_image(self, obj):
        return FileFieldListSerailizer(obj.image, many=False, read_only=True, context=self.context).data

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())


# SliderVideo
class SliderVideoDetailsSerializer(SliderDetailsSerializer):

    video = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SliderVideoDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SliderVideo
        fields = SliderDetailsSerializer.Meta.fields + [
            'video',
        ]

    def get_video(self, obj):
        return FileFieldListSerailizer(obj.video, many=False, read_only=True, context=self.context).data


class SliderSocialNetworkDetailsSerializer(SliderDetailsSerializer):

    social = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SliderSocialNetworkDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SliderSocialNetwork
        fields = SliderDetailsSerializer.Meta.fields + [
            'social',
        ]

    def get_social(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.social, many=False, read_only=True, context=self.context).data


# SliderLink
class SliderLinkNetworkDetailsSerializer(SliderDetailsSerializer):

    def __init__(self, *args, **kwargs):
        super(SliderLinkNetworkDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SliderLink
        fields = SliderDetailsSerializer.Meta.fields + [
            'link',
        ]


# SliderBaseModel
class SliderBaseModelDetailsSerializer(SliderDetailsSerializer):
    model = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SliderBaseModelDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SliderBaseModel
        fields = SliderDetailsSerializer.Meta.fields + [
            'model',
        ]

    def get_model(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.model, many=False, read_only=True, context=self.context).data


class SliderDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        self.model_serializer_mapping = {
            Slider: SliderDetailsSerializer,
            SliderVideo: SliderVideoDetailsSerializer,
            SliderSocialNetwork: SliderSocialNetworkDetailsSerializer,
            SliderLink: SliderLinkNetworkDetailsSerializer,
            SliderBaseModel: SliderBaseModelDetailsSerializer
        }
        super(SliderDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import SliderSegment

    # Base Segment ListSegmentReview
    class SegmentSliderListSerializer(BaseSegmentListSerializer):

        class Meta:
            model = SliderSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Segmt Details
    class SegmentSliderDetailSerializer(BaseSegmentDetailSerializer, SegmentSliderListSerializer):

        class Meta:
            model = SliderSegment
            fields = SegmentSliderListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentSliderListSerializer.Meta.read_only_fields + []



from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from aparnik.contrib.reviews.api.serializers import BaseReviewDetailsSerializer, BaseReviewListSerializer
from ..models import QA
from aparnik.contrib.basemodels.models import BaseModel


# QA List Serializer
class QAListSerializer(BaseReviewListSerializer):
    files = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(QAListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = QA
        fields = BaseReviewListSerializer.Meta.fields + [
            'files',
        ]
        read_only_fields = BaseReviewListSerializer.Meta.read_only_fields + []

    def get_files(self, obj):
        from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
        return FileFieldListSerailizer(obj.files, many=True, read_only=True, context=self.context).data if not None else None


# QA Details Serializer
class QADetailSerializer(QAListSerializer, BaseReviewDetailsSerializer):

    def __init__(self, *args, **kwargs):
        super(QADetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = QA
        fields = QAListSerializer.Meta.fields + BaseReviewDetailsSerializer.Meta.fields + [

        ]
        read_only_fields = QAListSerializer.Meta.read_only_fields + BaseReviewDetailsSerializer.Meta.read_only_fields + []


# QA Create Serializer
class QACreateSerializer(QADetailSerializer):
    model_obj_id = serializers.IntegerField(write_only=True, required=True)
    parent_obj_id = serializers.IntegerField(write_only=True, required=False)
    resourcetype = serializers.SerializerMethodField()

    class Meta:
        model = QA
        fields = QADetailSerializer.Meta.fields + [
            'model_obj_id',
            'parent_obj_id',
            'resourcetype',

        ]
        read_only_fields = QADetailSerializer.Meta.read_only_fields + []

    def validate_model_obj_id(self, value):
        try:
            basemodel = get_object_or_404(BaseModel.objects.all(), id=value)
            return value
        except Exception as e:
            raise serializers.ValidationError(_("Base Model not found"))

    def validate_parent_obj_id(self, value):
        try:
            qa = get_object_or_404(QA.objects.all(), id=value)
            return value
        except Exception as e:
            raise serializers.ValidationError(_("QA Parent not found"))

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_resourcetype(self, obj):
        return 'QA'



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Q
from django.contrib.auth import get_user_model
from django.core.validators import ValidationError
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.settings.models import Setting
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager


User = get_user_model()


# Product Sharing
class ProductSharingManager(BaseModelManager):
    def get_queryset(self):
        return super(ProductSharingManager, self).get_queryset()

    def active(self):
        return super(ProductSharingManager, self).active().filter(is_active=True)

    def share_this_user(self, user, product_id=None):
        if not user.is_authenticated:
            return ProductSharing.objects.none()

        # TODO: اضافه کردن این که فقط فعالین را بر گرداند یا خیر !
        queryset = self.active().filter(user_obj=user)
        if product_id:
            queryset = queryset.filter(product_obj__id=product_id)
        return queryset

    def share_with_this_user(self, user, product_id=None):
        queryset = self.active().filter(ProductSharing.query_share_with(user))
        if product_id:
            queryset = queryset.filter(product_obj__id=product_id)
        return queryset


class ProductSharing(BaseModel):

    user_obj = models.ForeignKey(User, related_name='productsharing_user_source', on_delete=models.CASCADE, verbose_name=_('User'))
    user_product_share_with_obj = models.ForeignKey(User, related_name='productsharing_user_product_share_with', on_delete=models.CASCADE, verbose_name=_('Share With'))
    product_obj = models.ForeignKey('products.Product', related_name='productsharing_product', on_delete=models.CASCADE, verbose_name=_('Product'))
    is_active = models.BooleanField(default=False, verbose_name=_('Is Active'))

    objects = ProductSharingManager()

    class Meta:
        verbose_name = _('Product Sharing')
        verbose_name_plural = _('Products Sharing')

    def __str__(self):
        return str(self.user_obj)

    def clean(self):

        if self.user_obj == self.user_product_share_with_obj:
            # both user are the same
            raise ValidationError({
                'user_obj': [_('Both can not have the same values.'), ],
                'user_product_share_with_obj': [_('Both can not have the same values.'), ]
            })
        if self.is_active:
            if not self.product_obj.is_buy(user=self.user_obj):
                raise ValidationError({
                    'user_obj': [_('You must first purchase this item.'), ],
                })

            # بررسی این که این کاربر دسترسی دارد یا خیر
            if self.product_obj.has_permit(user=self.user_product_share_with_obj):
                raise ValidationError({
                    'user_product_share_with_obj': [_('This user already has access to this product.'), ],
                })

            # بررسی دسترسی های داده شده توسط این کاربر برای یک کالای خاص
            max_share_per_product = Setting.objects.get(key='MAX_PRODUCT_SHARING_USER_ALLOWED_PER_PRODUCT').get_value()
            if ProductSharing.objects.share_this_user(product_id=self.product_obj.id, user=self.user_obj).count() > max_share_per_product:
                raise ValidationError({
                    'user_obj': [
                        _('The ceiling for the number of accesses that you could have made for this item is over. Please contact the support team.')
                    ],
                })

            # بررسی کل دسترسی های داده شده توسط این کاربر.
            max_share_for_all_products = Setting.objects.get(key='MAX_PRODUCT_SHARING_USER_ALLOWED_FOR_ALL_PRODUCTS').get_value()
            if ProductSharing.objects.share_this_user(user=self.user_obj).count() > max_share_for_all_products:
                raise ValidationError({
                    'user_obj': [
                        _('The ceiling of the number of accesses you could have completed is over. Please contact the support team.'), ],
                })

        return super(ProductSharing, self).clean()

    def save(self, *args, **kwargs):
        self.full_clean()
        if self.is_active:
            from aparnik.contrib.notifications.models import Notification
            Notification.objects.send_notification(
                users=[self.user_product_share_with_obj],
                from_user_obj=self.user_obj,
                model_obj=self.product_obj,
                type=Notification.NOTIFICATION_INFO,
                title='اضافه شدن دسترسی',
                description='دسترسی شما به %s توسط %s اضافه شد.' %(self.product_obj.title, self.user_obj.last_name),
            )

        return super(ProductSharing, self).save(*args, **kwargs)

    @staticmethod
    def query_share_with(user, prefix=''):
        field_share_with = 'user_product_share_with_obj'
        if prefix:
            field_share_with = prefix + '__' + field_share_with

        query_filter = Q(**{field_share_with: user})

        return query_filter


from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer

from aparnik.contrib.shortblogs.api.serializers import ShortBlogDetailSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from aparnik.contrib.segments.api.serializers import BaseSegmentListPolymorphicSerializer
from ..models import Information


class AboutusDestailSerializer(ModelSerializer):

    socials = serializers.SerializerMethodField()
    short_blogs = serializers.SerializerMethodField()
    image = serializers.SerializerMethodField()
    slider_segment = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(AboutusDestailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Information
        fields = [
            'id',
            'website',
            'address',
            'phone',
            'email',
            'about_us',
            'image',
            'socials',
            'short_blogs',
            'slider_segment',
        ]

    def get_socials(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.socials, many=True, read_only=True, context=self.context).data

    def get_short_blogs(self, obj):
        return ShortBlogDetailSerializer(obj.short_blogs, many=True, read_only=True, context=self.context).data

    def get_image(self, obj):
        return FileFieldListSerailizer(obj.image, many=False, read_only=True, context=self.context).data

    def get_slider_segment(self, obj):
        return BaseSegmentListPolymorphicSerializer(obj.slider_segment_obj, many=False, read_only=True, context=self.context).data if obj.slider_segment_obj else None

